import enum
import os
try:
    from aiges_embed import Response, DataListCls, SessionCreateResponse  # c++
except:
    from aiges.dto import Response, DataListCls, SessionCreateResponse

from ailab.atp_finetuner.constant import Model
from aiges.utils.log import log

class WTaskType(enum.Enum):
    NLP = 'nlp'
    NLP_VLLM = 'nlp_vllm'
    CV_IMAGE_CLASSIFICATION = 'image_classification'
    CV_OBJECT_DETECTION = 'object_detection'
    TEXT_TO_IMAGE = 'diffusers_text_to_image'

def GetTaskType(model: str) -> WTaskType:
    diffusers_text_to_image_model = [Model.stable_diffusion_1_4, Model.stable_diffusion_2_1,Model.stable_diffusion_xl_base_1]
    image_classification_model = [Model.vit_patch16_224_in21k]
    object_detection_model = [Model.yolos_base, Model.yolos_small]
    vllm_models = [Model.llama2_7b, Model.chinese_alpaca_2, Model.codellama_70b_instruction, Model.phind_codellama_34b_v2]

    if model in vllm_models:
        return WTaskType.NLP_VLLM
    elif model in diffusers_text_to_image_model:
        return WTaskType.TEXT_TO_IMAGE
    elif model in image_classification_model:
        return WTaskType.CV_IMAGE_CLASSIFICATION
    elif model in object_detection_model:
        return WTaskType.CV_OBJECT_DETECTION
    else:
        return WTaskType.NLP

class Wrapper:
    def __init__(self, *args, **kwargs):
        pretrained_model = os.environ.get("PRETRAINED_MODEL_NAME")
        log.info(f"Wrapper init pretrained_model:{pretrained_model}")
        if pretrained_model is None:
            raise ValueError("pretrained_model env not set")
        from ailab.utils.model_depend import install_model_depend
        task = GetTaskType(pretrained_model)
        log.info(f"Wrapper init task:{task}")
        install_model_depend(pretrained_model, is_inference=True, use_vllm=(task == WTaskType.NLP_VLLM))
        log.info(f"install depends finished")
        if task == WTaskType.NLP_VLLM:
            log.info(f"vllm infer...")
            from ailab.inference_wrapper.huggingface.lora.nlp.wrapper_vllm import Wrapper as OlmWrapper
        elif task == WTaskType.NLP:
            from ailab.inference_wrapper.huggingface.lora.nlp.wrapper import Wrapper as OlmWrapper
        elif task == WTaskType.TEXT_TO_IMAGE:
            from ailab.inference_wrapper.huggingface.diffusers.stable_diffusion.text_to_image.wrapper.wrapper import Wrapper as OlmWrapper
        elif task == WTaskType.CV_IMAGE_CLASSIFICATION:
            from ailab.inference_wrapper.huggingface.transformers.cv.image_classification.wrapper.wrapper_full import Wrapper as OlmWrapper
        elif task == WTaskType.CV_OBJECT_DETECTION:
            from ailab.inference_wrapper.huggingface.transformers.cv.object_detection.wrapper.wrapper_full import Wrapper as OlmWrapper
        self._wrapper = OlmWrapper(*args, **kwargs)

    def wrapperInit(self, config: {}) -> int:
        return self._wrapper.wrapperInit(config)

    def wrapperLoadRes(self, reqData: DataListCls, patch_id: int) -> int:
        return self._wrapper.wrapperLoadRes(reqData, patch_id)
    
    def wrapperUnloadRes(self, patch_id: int) -> int:
        return self._wrapper.wrapperUnloadRes(patch_id)
    
    def wrapperOnceExec(self, params: {}, reqData: DataListCls, usrTag:str="",persId: int = 0) -> Response:
        return self._wrapper.wrapperOnceExec(params, reqData, usrTag, persId)

    def wrapperOnceExecAsync(self, params: {}, reqData: DataListCls, usrTag: str, persId: int = 0) -> int:
        return self._wrapper.wrapperOnceExecAsync(params, reqData, usrTag, persId)

    def wrapperFini(self) -> int:
        return self._wrapper.wrapperFini()

    def wrapperError(self, ret: int) -> str:
        return self._wrapper.wrapperError()
    
    # def wrapperWrite(self, handle: str, req: DataListCls sid: str) -> int:
    def wrapperWrite(self, handle: str, req: DataListCls) -> int:
        return self._wrapper.wrapperWrite(handle, req)

    def wrapperCreate(self, params: {}, sid: str, persId: int = 0, usrTag: str="") -> SessionCreateResponse:
        return self._wrapper.wrapperCreate(params, sid, persId, usrTag)

    def wrapperDestroy(self, handle: str) -> int:
        return self._wrapper.wrapperDestroy(handle)

    def wrapperTestFunc(self, data: [], respData: []):
        return self._wrapper.wrapperTestFunc(data, respData)